/*
 * Copyright 2010-2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.amazonaws.services.iot.model;

import java.io.Serializable;

/**
 * <p>
 * Describes an action that sends data to CloudWatch Logs.
 * </p>
 */
public class CloudwatchLogsAction implements Serializable {
    /**
     * <p>
     * The IAM role that allows access to the CloudWatch log.
     * </p>
     */
    private String roleArn;

    /**
     * <p>
     * The CloudWatch log group to which the action sends data.
     * </p>
     */
    private String logGroupName;

    /**
     * <p>
     * Indicates whether batches of log records will be extracted and uploaded
     * into CloudWatch. Values include <code>true</code> or <code>false</code>
     * <i>(default)</i>.
     * </p>
     */
    private Boolean batchMode;

    /**
     * <p>
     * The IAM role that allows access to the CloudWatch log.
     * </p>
     *
     * @return <p>
     *         The IAM role that allows access to the CloudWatch log.
     *         </p>
     */
    public String getRoleArn() {
        return roleArn;
    }

    /**
     * <p>
     * The IAM role that allows access to the CloudWatch log.
     * </p>
     *
     * @param roleArn <p>
     *            The IAM role that allows access to the CloudWatch log.
     *            </p>
     */
    public void setRoleArn(String roleArn) {
        this.roleArn = roleArn;
    }

    /**
     * <p>
     * The IAM role that allows access to the CloudWatch log.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     *
     * @param roleArn <p>
     *            The IAM role that allows access to the CloudWatch log.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public CloudwatchLogsAction withRoleArn(String roleArn) {
        this.roleArn = roleArn;
        return this;
    }

    /**
     * <p>
     * The CloudWatch log group to which the action sends data.
     * </p>
     *
     * @return <p>
     *         The CloudWatch log group to which the action sends data.
     *         </p>
     */
    public String getLogGroupName() {
        return logGroupName;
    }

    /**
     * <p>
     * The CloudWatch log group to which the action sends data.
     * </p>
     *
     * @param logGroupName <p>
     *            The CloudWatch log group to which the action sends data.
     *            </p>
     */
    public void setLogGroupName(String logGroupName) {
        this.logGroupName = logGroupName;
    }

    /**
     * <p>
     * The CloudWatch log group to which the action sends data.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     *
     * @param logGroupName <p>
     *            The CloudWatch log group to which the action sends data.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public CloudwatchLogsAction withLogGroupName(String logGroupName) {
        this.logGroupName = logGroupName;
        return this;
    }

    /**
     * <p>
     * Indicates whether batches of log records will be extracted and uploaded
     * into CloudWatch. Values include <code>true</code> or <code>false</code>
     * <i>(default)</i>.
     * </p>
     *
     * @return <p>
     *         Indicates whether batches of log records will be extracted and
     *         uploaded into CloudWatch. Values include <code>true</code> or
     *         <code>false</code> <i>(default)</i>.
     *         </p>
     */
    public Boolean isBatchMode() {
        return batchMode;
    }

    /**
     * <p>
     * Indicates whether batches of log records will be extracted and uploaded
     * into CloudWatch. Values include <code>true</code> or <code>false</code>
     * <i>(default)</i>.
     * </p>
     *
     * @return <p>
     *         Indicates whether batches of log records will be extracted and
     *         uploaded into CloudWatch. Values include <code>true</code> or
     *         <code>false</code> <i>(default)</i>.
     *         </p>
     */
    public Boolean getBatchMode() {
        return batchMode;
    }

    /**
     * <p>
     * Indicates whether batches of log records will be extracted and uploaded
     * into CloudWatch. Values include <code>true</code> or <code>false</code>
     * <i>(default)</i>.
     * </p>
     *
     * @param batchMode <p>
     *            Indicates whether batches of log records will be extracted and
     *            uploaded into CloudWatch. Values include <code>true</code> or
     *            <code>false</code> <i>(default)</i>.
     *            </p>
     */
    public void setBatchMode(Boolean batchMode) {
        this.batchMode = batchMode;
    }

    /**
     * <p>
     * Indicates whether batches of log records will be extracted and uploaded
     * into CloudWatch. Values include <code>true</code> or <code>false</code>
     * <i>(default)</i>.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     *
     * @param batchMode <p>
     *            Indicates whether batches of log records will be extracted and
     *            uploaded into CloudWatch. Values include <code>true</code> or
     *            <code>false</code> <i>(default)</i>.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public CloudwatchLogsAction withBatchMode(Boolean batchMode) {
        this.batchMode = batchMode;
        return this;
    }

    /**
     * Returns a string representation of this object; useful for testing and
     * debugging.
     *
     * @return A string representation of this object.
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        if (getRoleArn() != null)
            sb.append("roleArn: " + getRoleArn() + ",");
        if (getLogGroupName() != null)
            sb.append("logGroupName: " + getLogGroupName() + ",");
        if (getBatchMode() != null)
            sb.append("batchMode: " + getBatchMode());
        sb.append("}");
        return sb.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int hashCode = 1;

        hashCode = prime * hashCode + ((getRoleArn() == null) ? 0 : getRoleArn().hashCode());
        hashCode = prime * hashCode
                + ((getLogGroupName() == null) ? 0 : getLogGroupName().hashCode());
        hashCode = prime * hashCode + ((getBatchMode() == null) ? 0 : getBatchMode().hashCode());
        return hashCode;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;

        if (obj instanceof CloudwatchLogsAction == false)
            return false;
        CloudwatchLogsAction other = (CloudwatchLogsAction) obj;

        if (other.getRoleArn() == null ^ this.getRoleArn() == null)
            return false;
        if (other.getRoleArn() != null && other.getRoleArn().equals(this.getRoleArn()) == false)
            return false;
        if (other.getLogGroupName() == null ^ this.getLogGroupName() == null)
            return false;
        if (other.getLogGroupName() != null
                && other.getLogGroupName().equals(this.getLogGroupName()) == false)
            return false;
        if (other.getBatchMode() == null ^ this.getBatchMode() == null)
            return false;
        if (other.getBatchMode() != null
                && other.getBatchMode().equals(this.getBatchMode()) == false)
            return false;
        return true;
    }
}
