/*
 * Copyright 2010-2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.amazonaws.services.rekognition.model;

import java.io.Serializable;

/**
 * <p>
 * Sets up the flow definition the image will be sent to if one of the
 * conditions is met. You can also set certain attributes of the image before
 * review.
 * </p>
 */
public class HumanLoopConfig implements Serializable {
    /**
     * <p>
     * The name of the human review used for this image. This should be kept
     * unique within a region.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 63<br/>
     * <b>Pattern: </b>^[a-z0-9](-*[a-z0-9])*<br/>
     */
    private String humanLoopName;

    /**
     * <p>
     * The Amazon Resource Name (ARN) of the flow definition. You can create a
     * flow definition by using the Amazon Sagemaker <a href=
     * "https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateFlowDefinition.html"
     * >CreateFlowDefinition</a> Operation.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b> - 256<br/>
     */
    private String flowDefinitionArn;

    /**
     * <p>
     * Sets attributes of the input data.
     * </p>
     */
    private HumanLoopDataAttributes dataAttributes;

    /**
     * <p>
     * The name of the human review used for this image. This should be kept
     * unique within a region.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 63<br/>
     * <b>Pattern: </b>^[a-z0-9](-*[a-z0-9])*<br/>
     *
     * @return <p>
     *         The name of the human review used for this image. This should be
     *         kept unique within a region.
     *         </p>
     */
    public String getHumanLoopName() {
        return humanLoopName;
    }

    /**
     * <p>
     * The name of the human review used for this image. This should be kept
     * unique within a region.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 63<br/>
     * <b>Pattern: </b>^[a-z0-9](-*[a-z0-9])*<br/>
     *
     * @param humanLoopName <p>
     *            The name of the human review used for this image. This should
     *            be kept unique within a region.
     *            </p>
     */
    public void setHumanLoopName(String humanLoopName) {
        this.humanLoopName = humanLoopName;
    }

    /**
     * <p>
     * The name of the human review used for this image. This should be kept
     * unique within a region.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b>1 - 63<br/>
     * <b>Pattern: </b>^[a-z0-9](-*[a-z0-9])*<br/>
     *
     * @param humanLoopName <p>
     *            The name of the human review used for this image. This should
     *            be kept unique within a region.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public HumanLoopConfig withHumanLoopName(String humanLoopName) {
        this.humanLoopName = humanLoopName;
        return this;
    }

    /**
     * <p>
     * The Amazon Resource Name (ARN) of the flow definition. You can create a
     * flow definition by using the Amazon Sagemaker <a href=
     * "https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateFlowDefinition.html"
     * >CreateFlowDefinition</a> Operation.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b> - 256<br/>
     *
     * @return <p>
     *         The Amazon Resource Name (ARN) of the flow definition. You can
     *         create a flow definition by using the Amazon Sagemaker <a href=
     *         "https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateFlowDefinition.html"
     *         >CreateFlowDefinition</a> Operation.
     *         </p>
     */
    public String getFlowDefinitionArn() {
        return flowDefinitionArn;
    }

    /**
     * <p>
     * The Amazon Resource Name (ARN) of the flow definition. You can create a
     * flow definition by using the Amazon Sagemaker <a href=
     * "https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateFlowDefinition.html"
     * >CreateFlowDefinition</a> Operation.
     * </p>
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b> - 256<br/>
     *
     * @param flowDefinitionArn <p>
     *            The Amazon Resource Name (ARN) of the flow definition. You can
     *            create a flow definition by using the Amazon Sagemaker <a
     *            href=
     *            "https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateFlowDefinition.html"
     *            >CreateFlowDefinition</a> Operation.
     *            </p>
     */
    public void setFlowDefinitionArn(String flowDefinitionArn) {
        this.flowDefinitionArn = flowDefinitionArn;
    }

    /**
     * <p>
     * The Amazon Resource Name (ARN) of the flow definition. You can create a
     * flow definition by using the Amazon Sagemaker <a href=
     * "https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateFlowDefinition.html"
     * >CreateFlowDefinition</a> Operation.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     * <p>
     * <b>Constraints:</b><br/>
     * <b>Length: </b> - 256<br/>
     *
     * @param flowDefinitionArn <p>
     *            The Amazon Resource Name (ARN) of the flow definition. You can
     *            create a flow definition by using the Amazon Sagemaker <a
     *            href=
     *            "https://docs.aws.amazon.com/sagemaker/latest/dg/API_CreateFlowDefinition.html"
     *            >CreateFlowDefinition</a> Operation.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public HumanLoopConfig withFlowDefinitionArn(String flowDefinitionArn) {
        this.flowDefinitionArn = flowDefinitionArn;
        return this;
    }

    /**
     * <p>
     * Sets attributes of the input data.
     * </p>
     *
     * @return <p>
     *         Sets attributes of the input data.
     *         </p>
     */
    public HumanLoopDataAttributes getDataAttributes() {
        return dataAttributes;
    }

    /**
     * <p>
     * Sets attributes of the input data.
     * </p>
     *
     * @param dataAttributes <p>
     *            Sets attributes of the input data.
     *            </p>
     */
    public void setDataAttributes(HumanLoopDataAttributes dataAttributes) {
        this.dataAttributes = dataAttributes;
    }

    /**
     * <p>
     * Sets attributes of the input data.
     * </p>
     * <p>
     * Returns a reference to this object so that method calls can be chained
     * together.
     *
     * @param dataAttributes <p>
     *            Sets attributes of the input data.
     *            </p>
     * @return A reference to this updated object so that method calls can be
     *         chained together.
     */
    public HumanLoopConfig withDataAttributes(HumanLoopDataAttributes dataAttributes) {
        this.dataAttributes = dataAttributes;
        return this;
    }

    /**
     * Returns a string representation of this object; useful for testing and
     * debugging.
     *
     * @return A string representation of this object.
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        if (getHumanLoopName() != null)
            sb.append("HumanLoopName: " + getHumanLoopName() + ",");
        if (getFlowDefinitionArn() != null)
            sb.append("FlowDefinitionArn: " + getFlowDefinitionArn() + ",");
        if (getDataAttributes() != null)
            sb.append("DataAttributes: " + getDataAttributes());
        sb.append("}");
        return sb.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int hashCode = 1;

        hashCode = prime * hashCode
                + ((getHumanLoopName() == null) ? 0 : getHumanLoopName().hashCode());
        hashCode = prime * hashCode
                + ((getFlowDefinitionArn() == null) ? 0 : getFlowDefinitionArn().hashCode());
        hashCode = prime * hashCode
                + ((getDataAttributes() == null) ? 0 : getDataAttributes().hashCode());
        return hashCode;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;

        if (obj instanceof HumanLoopConfig == false)
            return false;
        HumanLoopConfig other = (HumanLoopConfig) obj;

        if (other.getHumanLoopName() == null ^ this.getHumanLoopName() == null)
            return false;
        if (other.getHumanLoopName() != null
                && other.getHumanLoopName().equals(this.getHumanLoopName()) == false)
            return false;
        if (other.getFlowDefinitionArn() == null ^ this.getFlowDefinitionArn() == null)
            return false;
        if (other.getFlowDefinitionArn() != null
                && other.getFlowDefinitionArn().equals(this.getFlowDefinitionArn()) == false)
            return false;
        if (other.getDataAttributes() == null ^ this.getDataAttributes() == null)
            return false;
        if (other.getDataAttributes() != null
                && other.getDataAttributes().equals(this.getDataAttributes()) == false)
            return false;
        return true;
    }
}
